@@ -28,6 +28,7 @@ GAME_MAP = 'game_map'
 SKIP_REPLAYS = 'skip_replays'
 INSTANT_START = 'start_without_countdown'
 EXISTING_MATCH_BEHAVIOR = 'existing_match_behavior'
+ENABLE_LOCKSTEP = 'enable_lockstep'
 
 logger = get_logger('config_parser')
 
@@ -37,6 +38,7 @@ game_mode_types = [
     "Dropshot",
     "Hockey",
     "Rumble",
+    "Heatseeker",
 ]
 
 map_types = [
@@ -77,7 +79,11 @@ map_types = [
     "UtopiaRetro",
     "Hoops_DunkHouse",
     "DropShot_Core707",
-    "ThrowbackStadium"
+    "ThrowbackStadium",
+    "ForbiddenTemple",
+    "RivalsArena",
+    "Farmstead_Night",
+    "SaltyShores_Night"
 ]
 
 match_length_types = [
@@ -209,7 +215,7 @@ def add_match_settings_header(config_object):
                            description='Number of bots/players which will be spawned.  We support up to max 64.')
     match_header.add_value(GAME_MODE, str, default="Soccer",
                            description="""What game mode the game should load.
-                           Accepted values are "Soccer", "Hoops", "Dropshot", "Hockey", "Rumble" """)
+                           Accepted values are "Soccer", "Hoops", "Dropshot", "Hockey", "Rumble", "Heatseeker" """)
     match_header.add_value(GAME_MAP, str, default="DFHStadium",
                            description="""Which map the game should load into. Too many to list.""")
     match_header.add_value(SKIP_REPLAYS, bool, default=False,
@@ -218,6 +224,8 @@ def add_match_settings_header(config_object):
                            description="""Skip the kickoff countdown""")
     match_header.add_value(EXISTING_MATCH_BEHAVIOR, str, default="Restart If Different",
                            description="""What should we do if you click run while a match is already in progress?""")
+    match_header.add_value(ENABLE_LOCKSTEP, bool, default=False,
+                           description="""If True, the framework will wait for outputs from all bots before advancing to the next frame.""")
 
 
 def add_mutator_header(config_object):
@@ -312,5 +320,6 @@ def parse_match_settings(match_settings, config: ConfigObject):
     match_settings.skip_replays = config.getboolean(MATCH_CONFIGURATION_HEADER, SKIP_REPLAYS)
     match_settings.instant_start = config.getboolean(MATCH_CONFIGURATION_HEADER, INSTANT_START)
     match_settings.existing_match_behavior = config.get(MATCH_CONFIGURATION_HEADER, EXISTING_MATCH_BEHAVIOR)
+    match_settings.enable_lockstep = config.getboolean(MATCH_CONFIGURATION_HEADER, ENABLE_LOCKSTEP)
 
     parse_mutator_settings(match_settings.mutators, config)
