@@ -2,14 +2,16 @@ import ctypes
 import os
 import sys
 import time
+import platform
 from logging import DEBUG, WARNING
 
 import flatbuffers
+from rlbot.messages.flat.MatchSettings import MatchSettings as MatchSettingsPacket
 from rlbot.messages.flat.BallPrediction import BallPrediction as BallPredictionPacket
 from rlbot.messages.flat.FieldInfo import FieldInfo
 from rlbot.messages.flat.QuickChatMessages import QuickChatMessages
 from rlbot.utils.file_util import get_python_root
-from rlbot.utils.game_state_util import GameState
+from rlbot.utils.game_state_util import GameState, CarState, Physics, Vector3
 from rlbot.utils.rendering.rendering_manager import RenderingManager
 from rlbot.utils.rlbot_exception import get_exception_from_error_code, EmptyDllResponse
 from rlbot.utils.structures import game_data_struct
@@ -20,6 +22,15 @@ from rlbot.utils.structures.game_status import RLBotCoreStatus
 from rlbot.utils.structures.rigid_body_struct import RigidBodyTick
 from rlbot.utils.structures.start_match_structures import MatchSettings
 
+if platform.system() == 'Windows':
+    dll_name_64 = 'RLBot_Core_Interface.dll'
+    dll_name_32 = 'RLBot_Core_Interface_32.dll'
+elif platform.system() == 'Linux':
+    dll_name_64 = 'libRLBotInterface.so'
+    dll_name_32 = 'libRLBotInterface32.so'
+elif platform.system() == 'Darwin':
+    dll_name_64 = 'libRLBotInterface.dylib'
+    dll_name_32 = 'libRLBotInterface32.dylib'
 
 def wrap_callback(callback_func):
     def caller(id, status):
@@ -29,11 +40,11 @@ def wrap_callback(callback_func):
 
 
 def get_dll_location():
-    return os.path.join(get_dll_directory(), 'RLBot_Core_Interface.dll')
+    return os.path.join(get_dll_directory(), dll_name_64)
 
 
 def get_dll_32_location():
-    return os.path.join(get_dll_directory(), 'RLBot_Core_Interface_32.dll')
+    return os.path.join(get_dll_directory(), dll_name_32)
 
 
 def is_32_bit_python():
@@ -121,6 +132,18 @@ class GameInterface:
         func.argtypes = [ctypes.c_void_p, ctypes.c_int]
         func.restype = ctypes.c_int
 
+        func = self.game.GetMatchSettings
+        func.argtypes = []
+        func.restype = ByteBuffer
+
+        func = self.game.FreshLiveDataPacket
+        func.argtypes = [ctypes.POINTER(GameTickPacket), ctypes.c_int, ctypes.c_int]
+        func.restype = ctypes.c_int
+
+        func = self.game.FreshLiveDataPacketFlatbuffer
+        func.argtypes = [ctypes.c_int, ctypes.c_int]
+        func.restype = ByteBuffer
+
         self.renderer.setup_function_types(self.game)
         self.logger.debug('game interface functions are setup')
 
@@ -133,6 +156,11 @@ class GameInterface:
         self.game_status(None, rlbot_status)
         return game_tick_packet
 
+    def fresh_live_data_packet(self, game_tick_packet: GameTickPacket, timeout_millis: int, key: int):
+        rlbot_status = self.game.FreshLiveDataPacket(game_tick_packet, timeout_millis, key)
+        self.game_status(None, rlbot_status)
+        return game_tick_packet
+
     def update_field_info_packet(self, field_info_packet: FieldInfoPacket):
         rlbot_status = self.game.UpdateFieldInfo(field_info_packet)
         self.game_status(None, rlbot_status)
@@ -185,29 +213,50 @@ class GameInterface:
             self.logger.log(level, "bad status %s", RLBotCoreStatus.status_list[rlbot_status])
 
     def wait_until_loaded(self):
-        num_iterations = 0
-        while num_iterations < 60:
+        for i in range(0, 120):
             self.game.IsInitialized.restype = ctypes.c_bool
             is_loaded = self.game.IsInitialized()
             if is_loaded:
                 self.logger.info('DLL is initialized!')
                 return
             else:
-                num_iterations += 1
                 time.sleep(1)
         raise TimeoutError("RLBot took too long to initialize! Was Rocket League started with the -rlbot flag? "
                            "If you're not sure, close Rocket League and let us open it for you next time!")
 
     def wait_until_valid_packet(self):
-        while True:
+        self.logger.info('Waiting for valid packet...')
+        for i in range(0, 60):
             packet = game_data_struct.GameTickPacket()
             self.update_live_data_packet(packet)
-            if packet.num_cars > 0:
-                self.logger.info('Packets are looking good!')
-                return
-            else:
-                self.logger.debug('Waiting for valid packet...')
-                time.sleep(0.5)
+            if not packet.game_info.is_match_ended:
+                spawn_ids = set()
+                for k in range(0, self.start_match_configuration.num_players):
+                    player_config = self.start_match_configuration.player_configuration[k]
+                    if player_config.rlbot_controlled:
+                        spawn_ids.add(player_config.spawn_id)
+
+                for n in range(0, packet.num_cars):
+                    try:
+                        spawn_ids.remove(packet.game_cars[n].spawn_id)
+                    except KeyError:
+                        pass
+
+                if len(spawn_ids) == 0:
+                    self.logger.info('Packets are looking good, all spawn ids accounted for!')
+                    return
+                elif i > 4:
+                    car_states = {}
+                    for k in range(0, self.start_match_configuration.num_players):
+                        player_info = packet.game_cars[k]
+                        if player_info.spawn_id > 0:
+                            car_states[k] = CarState(physics=Physics(velocity=Vector3(z=500)))
+                    if len(car_states) > 0:
+                        self.logger.info("Scooting bots out of the way to allow more to spawn!")
+                        self.set_game_state(GameState(cars=car_states))
+
+            time.sleep(0.5)
+        self.logger.info('Gave up waiting for valid packet :(')
 
     def load_interface(self):
         self.game_status_callback_type = ctypes.CFUNCTYPE(None, ctypes.c_uint, ctypes.c_uint)
@@ -259,6 +308,22 @@ class GameInterface:
         data recorder that lives inside the core dll.
         """
         byte_buffer = self.game.UpdateLiveDataPacketFlatbuffer()
+        return self._process_live_flatbuffer(byte_buffer)
+
+    def get_fresh_live_data_flat_binary(self, timeout_millis: int, key: int):
+        """
+        Gets the live data packet in flatbuffer binary format. You'll need to do something like
+        GameTickPacket.GetRootAsGameTickPacket(binary, 0) to get the data out.
+        This one blocks until a fresh frame is available, or until the timeout has elapsed.
+        :param timeout_millis: will give up waiting for a fresh packet and just return a stale one
+        after this number of milliseconds.
+        :param key: answers the question "fresh from whose perspective?". Freshness will be
+        tracked separately for whatever key you pass. Bot index is a reasonable choice.
+        """
+        byte_buffer = self.game.FreshLiveDataPacketFlatbuffer(timeout_millis, key)
+        return self._process_live_flatbuffer(byte_buffer)
+
+    def _process_live_flatbuffer(self, byte_buffer):
         if byte_buffer.size >= 4:  # GetRootAsGameTickPacket gets angry if the size is less than 4
             # We're counting on this copying the data over to a new memory location so that the original
             # pointer can be freed safely.
@@ -306,3 +371,18 @@ class GameInterface:
             self.game.Free(byte_buffer.ptr)  # Avoid a memory leak
             self.game_status(None, RLBotCoreStatus.Success)
             return BallPredictionPacket.GetRootAsBallPrediction(proto_string, 0)
+
+    def get_match_settings(self) -> MatchSettings:
+        """
+        Gets the current match settings in flatbuffer format. Useful for determining map, game mode, mutator settings,
+        etc.
+        """
+        byte_buffer = self.game.GetMatchSettings()
+
+        if byte_buffer.size >= 4:  # GetRootAsGameTickPacket gets angry if the size is less than 4
+            # We're counting on this copying the data over to a new memory location so that the original
+            # pointer can be freed safely.
+            proto_string = ctypes.string_at(byte_buffer.ptr, byte_buffer.size)
+            self.game.Free(byte_buffer.ptr)  # Avoid a memory leak
+            self.game_status(None, RLBotCoreStatus.Success)
+            return MatchSettingsPacket.GetRootAsMatchSettings(proto_string, 0)
