@@ -4,18 +4,189 @@
 # 3) we can import it into your module module
 # https://stackoverflow.com/questions/458550/standard-way-to-embed-version-into-python-package
 
-__version__ = '1.20.1'
+__version__ = '1.40.0'
 
 release_notes = {
+    '1.40.0': """
+    Support for Heatseeker game mode. No ball prediction support for it yet, so good luck!
+    """,
+    '1.39.2': """
+    You can now generate a fresh loadout for your bot every match using a python script!
+    Try setting different colors based on your player index, or a randomized hat.
+    Learn how at https://github.com/RLBot/RLBot/wiki/Bot-Customization#loadout-generator
+
+    Also:
+    - Fixing color swatch translation. - Darxeal and tarehart
+    - Catching errors in hive bots. - Will
+    """,
+    '1.38.1': """
+    Ball prediction is now computed at 120Hz instead of 60Hz!
+    
+    Note that the data will still only have 60 slices in a second, but the positions will be more
+    accurate because of the extra precision behind the scenes. For example, bounces off the goal post
+    should be much more reliable now.
+    
+    Also:
+    - Fixed a bug where lockstep could prevent cars from spawning, and freeze forever.
+    - Fixed a bug where lockstep freezes forever if Psyonix bots are in the game.
+    """,
+    '1.37.2': """
+    - Better on screen performance measurement for RLBot (hit home key).
+    - Bots can now draw when the game is paused, and still move after goal explosions.
+    """,
+    '1.36.8': """
+    Proper appearances for Psyonix bots thanks to Marvin, with an assist from r0bbi3!
+
+    Also:
+    - Improved error messages. - Will
+    - Fixing bug when trying to spawn tons of cars. - tare
+    - Starting on hivemind support classes. Stay tuned! - Will
+    - Improving C# documentation. - Redox
+    - Feature for loading RLBot-specific game settings (e.g. framerate cap). - L0laapk3
+    - Fixed game_tick_packet.game_info.is_match_ended always false - Skyborg
+    - Fixed human spawning. - tare
+    - Making lockstep work smoother in Python. - tare
+    - Lockstep option in old GUI. - Redox
+    """,
+    '1.35.8': """
+    - Adding the Forbidden Temple stadium and a few others.
+    - Adding a few custom quick chats.
+    - Better support for loading levels that have no goals. - Skyborg
+    - Reducing the frequency of python hot reload checks. - Marvin
+    - Performance improvement related to logger. - Marvin
+    - C# documentation improvements. - Redox
+    - Providing a way to get autocomplete on the game tick packet.
+    - Trying again to fix inconsistencies with has_wheel_contact.
+    - Fixed bug that led to WrongProcessArgs error. - Will
+    - Adding a flag to control agent reloads for training. - Marvin
+    """,
+    '1.34.5': """
+    Mac support is here! https://github.com/RLBot/RLBot/wiki/Operating-System-Support
+
+    Also:
+    - Improving Remote RLBot so that multiple client computers can connect from behind
+      the same IP address.
+    - Fixing bug where bots would take a very long time to start if a human was in the
+      match config.
+    - Fixing input percentages and lockstep feature.
+    - Fix inconsistencies with has_wheel_contact. - Kipje13
+    """,
+    '1.33.1': """
+    Remote TCP functionality, primarily for classroom use.
+    See https://github.com/RLBot/RLBot/wiki/Remote-RLBot for details!
+
+    Minor updates:
+     - Fixing a bug that makes RLBot stutter unpredictably.
+    """,
+    '1.32.1': """
+    - You can specify your car's primary and secondary color swatches by the nearest
+      RGB value instead of looking up the color id's on our wiki. You're still constrained
+      to the normal color choices, don't get too excited!
+      See https://github.com/RLBot/RLBot/wiki/Bot-Customization#colors for details.
+    - The lockstep feature should be more reliable now.
+    """,
+    '1.31.1': """
+    Hitbox offsets are now available in the GameTickPacket.
+    This should make it possible to use hitboxes without hardcoding any values.
+
+    - Fixes hitboxes issues after RL update.
+    """,
+    '1.30.2': """
+    Kipje13 has a major speedup for rendering in python! You'll get the benefits
+    automatically, but ask him for the details if you're curious.
+
+    Also delivering fresher packets to python bots, for improved latency and
+    consistency. This may affect bot behavior slightly, check your kickoffs!
+    """,
+    '1.29.0': """
+    Python bots will now be able to specify their preferred tick rate. Do you like running at 120Hz?
+    You'll need to add "maximum_tick_rate_preference = 120" to your config file! Example:
+    https://github.com/RLBot/RLBot/blob/add379ab700d26f66ee31119a3b4e718a736f7aa/src/test/python/agents/atba/atba.cfg#L9
+    The default will be for python bots to run at 60, since that's how most bots have been developed and tested.
+    """,
+    '1.28.3': """
+    Linux Support!
+
+    Kipje13 has really done it, you can run RLBot in Linux now :D
+    - Tested in Ubuntu 18.04 with basic python bots, it works great.
+    - All the latest features like lockstep are present.
+    - This is early days, so many bots and languages won't work yet. Please think of this
+      as an alpha and help us find the bugs. Most of the remaining work will be in the
+      open source area of RLBot, so drop by #framework-dev if you want to contribute!
+
+    More info at https://github.com/RLBot/RLBot/wiki/Operating-System-Support
+
+    Additional fixes:
+    - Open RL using Popen on Linux - cominixo
+    - Fixed bug in Scratch manager and switched it to use structs.
+    - Ability to save scratch bots over the websocket.
+    """,
+    '1.27.1': """
+    Introducing lockstep mode--if you set enable_lockstep = True in rlbot.cfg, the
+    framework will wait for outputs from all bots before advancing to the next frame.
+
+    This feature brought to you by Redox!
+
+    Other features:
+    - No longer trying to configure Rocket League with .ini files, which was error-prone.
+    """,
+    '1.26.4': """
+    Adding the ability to execute arbitrary console commands in Rocket League.
+    See https://github.com/RLBot/RLBot/wiki/Console-Commands for details!
+
+    Other features:
+    - Adding a built-in version of the Scratch bot manager so we can push updates.
+    - Adding an early-start system you can opt into if your bot is very slow to start.
+      See https://github.com/RLBot/RLBot/wiki/Config-File-Documentation#early-start-system
+    - Scratch bots can optionally open a browser-per-bot and pretend to be player 1 on the
+      blue team. Good for tournaments!
+    - Adding back gravity and game speed setting. - Redox
+    """,
+    '1.25.5': """
+    - Added DummyRenderer, allowing its users to swallow all draw calls. - DomNomNom
+    - Fixed a bug where player names with special characters would cause crashes.
+    - Giving 'independent' bots more time to shut down.
+    - Fixed performance problem with python hot reload for large directories. - RamenAndVitamins
+    """,
+    '1.24.0': """
+    - Car hitbox data is now available for bots.
+    - The ball info now includes the shape (sphere, cube, etc.) and size of the ball.
+    - Hopefully reduces the amount of crashes when shutting down bot processes.
+    """,
+    '1.23.2': """
+    - Added goal dimensions to field info. - Kipje13
+    - Fixed bug where 'restart if different' doesn't start a match once the postgame timer expires.
+    - Setting current working directory to the executable location for java and .net bots.
+    - Bots can now include a logo! Either put a file called logo.png in the same folder as
+      your bot cfg, OR specify logo_file = ... in your bot cfg in the same section as python_file.
+      Dimensions of 400x300 are preferred. It will appear soon in RLBotGUI, and maybe on stream!
+    """,
+    '1.22.2': """
+    Fixed the bug that stopped many bots from working after the 1.22.1 update.
+    """,
+    '1.22.1': """
+    Bots can now retrieve the settings object used to launch the current match,
+    which can tell you the map, mode, mutator settings, etc.
+
+    Additional fixes:
+    - Waiting longer for rlbot to connect.
+    - Don't halt forever while waiting for valid packets.
+    - Python hot reload watches all your files.
+    """,
+    '1.21.0': """
+    - Latest ball touch now includes a player index value.
+    - Fixed blue team colors.
+    - Fixed loadout bug that caused incorrect antennas, etc.
+    """,
     '1.20.1': """
     Added support for the awesome new game mode Spike Rush!
-    
+
     To try it, set your mutators:
     Rumble = Spike Rush
     Respawn Time = 1 Second
-    
+
     To retract your spikes, you have use_item available in your controls output!
-    
+
     Additional fixes:
     - Will try harder to find an open port, to avoid 'std exception: listen' error.
     - Fixed a bug that prevented multiple .net bots from auto-starting. - Kipje13
@@ -35,15 +206,15 @@ release_notes = {
     Quickchat is back!
 
     Kipje13 added quickchat support deep in the framework so bots will be able to
-    read it across languages! Everything works exactly the same as before from the 
+    read it across languages! Everything works exactly the same as before from the
     bot maker's perspective. Stay tuned for instructions on how to read quick
     chat for languages like C# and Java. https://github.com/RLBot/RLBot/wiki/Quickchat
-    
+
     For now, the chat will use hacky rendering since we don't have official support yet.
-    
+
     For richer communication with coordinates etc, check out
     https://github.com/RLBot/RLBot/wiki/Matchcomms
-    
+
     Latest ball touch is also back!
     """,
     '1.16.5': """
